/*
  mem_move_eeprom.c
  
  This was my solution to storing data in EEPROM at specific
  memory locations, and having to break up LONGs,INTs, etc.;
  only to remap everything when I added another value to store.
  
  Example code to move large amounts of memory to/from
  EEPROM easily,  using a ONE-BYTE at a time method (doesn't
  work with FLASH if #bytes > sector).
  
  Same method can be used to transfer strings, structs, etc.
  to UART, SPI EEPROM, etc.
  
  This can be used on any PIC with EEPROM, or adapt the generic_eeprom.c
  for SPI EEPROM.
*/

#include "generic_eeprom.h"
const int EEPROM_STRUCT_LOC = 0X00;
const int EEPROM_ARY_LOC = 0XA0;
char etxt[]="This is some data to be stored";


/*
  This struct holds unit info thats loaded on power on.
  It is stored/retreived byte by byte into EEPROM.
  Anything you put in here will get saved with one command.
  This represents user parameters for some device that must be
  stored and retreived on power on. In order to determine if
  devicesd has NEVER been used, always put a HEADER BYTE in the struct
  to test.
*/
struct unit
{
    char header;             // header for validation (0xAA means struct valid)
    unsigned char address;   // unit address
    float soft_trip;         // software trip point
    unsigned dac_set;        // dac_setting
    char hilow_flag;         // flag (on/off)
    unsigned int hivolt_set; // high voltage output setpoint
    long fooLong;            //
    float fooFloat;          // analog output multiplier
    int fooInt;              //
    char txt[30];            // some string value
    char IP[16];             // more strings
};
struct unit Unit;            // declare an instance


/*
  Initialize
*/
void init_main (void){

    /*
      LOAD UNIT DATA FROM EEPROM AND CHECK IF VALID
      The first byte in struct is a test byte to see if
      this is the first time we powered up. If NOT valid
      we will load defualt values.  Note struct is filled
      with many unit types.  You can "reset to default" by
      changing to header byte & cycling power, so test fails.
    */
    EEPROM_GET((char*)&Unit,sizeof(Unit),EEPROM_STRUCT_LOC);
    /*
      Now the struct is loaded with the contents of the EEPROM!
      Use the debugger EEPROM window to verify.
    */
    asm nop;    // breakpoint to check STRUCT values
    asm nop;    // breakpoint to check STRUCT values
    // check for valid information
    if(Unit.header != 0xAA)       // check header byte first
    {
        Unit.header = 0xAA;       // set to 0xAA so next time its valid
        Unit.address = 99;
        Unit.soft_trip=0.000123;  // float value
        Unit.dac_set=512;
        Unit.hivolt_set = 24;     // char value
        Unit.fooFloat= 1e-9;      // diff notation
        Unit.fooLong = 123456;    //
        strcpy(Unit.txt,"Hello World I Love Mikroe!");
        strcpy(Unit.IP,"100.102.123.101");
        /*
          Now all default values loaded, we need to save it.
        */
        EEPROM_PUT((char*)&Unit,sizeof(Unit),EEPROM_STRUCT_LOC);  // store defaults
        asm nop;
    }
    asm nop;
}



/*
 MAIN()
 The init_main() is called, EEPROM loaded & tested.
 Whenever you change a paremeter that is stored, you
 can set a dirty flag, or immediately call the
 EEPROM_PUT to store the entire struct with one command.
*/
void main() 
{
      init_main();                   // intialize
      asm nop;    // breakpoint to check STRUCT values
      asm nop;    // breakpoint to check STRUCT values
      // now change a value & store
      strcpy(Unit.txt,"I HAVE BEEN CHANGED!");
      EEPROM_PUT((char*)&Unit,sizeof(Unit),EEPROM_STRUCT_LOC);  // store
      asm nop;    // breakpoint to check STRUCT values
      asm nop;    // breakpoint to check STRUCT values
      // reloaded and verify in EEPROM window
      EEPROM_GET((char*)&Unit,sizeof(Unit),EEPROM_STRUCT_LOC);
      asm nop;    // breakpoint to check STRUCT values
      asm nop;    // breakpoint to check STRUCT values
      /*
        now read/write the array
      */
      EEPROM_PUT((char*)&etxt,sizeof(etxt),EEPROM_ARY_LOC);  // store
      etxt[0]='Z';etxt[1]='Z';etxt[2]='Z';etxt[3]='Z';       // change a couple things
      asm nop;    // breakpoint to check STRUCT values
      asm nop;    // breakpoint to check STRUCT values
      EEPROM_GET((char*)&etxt,sizeof(etxt),EEPROM_ARY_LOC);  // restore
      asm nop;    // array in RAM is restored
      asm nop;    // breakpoint to check STRUCT values

      while(1);  // hang out
}